/*
 * Copyright (C) 2017-2024 Dominic Heutelbeck (dominic@heutelbeck.com)
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.sapl.grammar.ui;

import io.sapl.functions.FilterFunctionLibrary;
import io.sapl.functions.SchemaValidationLibrary;
import io.sapl.functions.StandardFunctionLibrary;
import io.sapl.functions.TemporalFunctionLibrary;
import io.sapl.grammar.ui.contentassist.SAPLUiContentProposalProvider;
import io.sapl.interpreter.InitializationException;
import io.sapl.pdp.config.PDPConfiguration;
import org.eclipse.xtext.ui.editor.contentassist.UiToIdeContentProposalProvider;
import org.junit.jupiter.api.Test;
import io.sapl.grammar.sapl.CombiningAlgorithm;

import java.util.Map;
import java.util.function.UnaryOperator;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import io.sapl.interpreter.combinators.PolicyDocumentCombiningAlgorithm;

class SAPLUiModuleTest {

    SAPLUiModule sut = new SAPLUiModule(null);

    @Test
    void testBindIContentProposalProvider() {
        assertThat(sut.bindIContentProposalProvider(), is(UiToIdeContentProposalProvider.class));
    }

    @Test
    void testBindIdeContentProposalProvider() {
        assertThat(sut.bindIdeContentProposalProvider(), is(SAPLUiContentProposalProvider.class));
    }

    @Test
    void testBindPDPConfigurationProvider() throws InitializationException {
        PDPConfiguration pdpConfiguration = sut.bindPDPConfigurationProvider().pdpConfiguration().blockFirst();

        assertThat(pdpConfiguration, is(not(nullValue())));
        assertThat(pdpConfiguration.isValid(), is(true));
        assertThat(pdpConfiguration.functionContext().getAvailableLibraries(), contains(FilterFunctionLibrary.NAME,
                StandardFunctionLibrary.NAME, TemporalFunctionLibrary.NAME, SchemaValidationLibrary.NAME));
        assertThat(pdpConfiguration.variables(), is(Map.of()));
        assertThat(pdpConfiguration.documentsCombinator(), is(PolicyDocumentCombiningAlgorithm.DENY_OVERRIDES));
        assertThat(pdpConfiguration.decisionInterceptorChain(), is(UnaryOperator.identity()));
        assertThat(pdpConfiguration.subscriptionInterceptorChain(), is(UnaryOperator.identity()));
    }
}
