/*
 * Copyright (C) 2017-2024 Dominic Heutelbeck (dominic@heutelbeck.com)
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.sapl.test.dsl.interpreter;

import io.sapl.api.interpreter.Val;
import io.sapl.test.SaplTestFixture;
import io.sapl.test.grammar.sapltest.Environment;
import io.sapl.test.steps.GivenOrWhenStep;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
class DefaultTestCaseConstructor {

    private final ValueInterpreter valueInterpreter;

    GivenOrWhenStep constructTestCase(final SaplTestFixture saplTestFixture, final Environment environment,
            final boolean needsMocks) {

        if (environment != null
                && environment.getEnvironment() instanceof io.sapl.test.grammar.sapltest.Object object) {
            final var environmentVariables = valueInterpreter.destructureObject(object);

            if (environmentVariables != null) {
                environmentVariables.forEach((key, json) -> saplTestFixture.registerVariable(key, Val.of(json)));
            }
        }

        return (GivenOrWhenStep) (needsMocks ? saplTestFixture.constructTestCaseWithMocks()
                : saplTestFixture.constructTestCase());
    }
}
